package xyz.pixelatedw.finalbeta.mixin;

import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfoReturnable;

import net.minecraft.entity.FurnaceEntity;
import net.minecraft.item.ItemInstance;
import net.minecraft.item.ItemType;
import net.minecraft.util.io.CompoundTag;
import xyz.pixelatedw.finalbeta.ModConfig;
import xyz.pixelatedw.finalbeta.ModTile;

@Mixin(FurnaceEntity.class)
public class FurnaceEntityMixin {

	@Shadow
	public int fuelTime;
	
	@Shadow
	private ItemInstance[] contents;
	
	@Inject(method = "tick", at = @At(value = "INVOKE", target = "Lnet/minecraft/entity/FurnaceEntity;getFuelTime(Lnet/minecraft/item/ItemInstance;)I", shift = At.Shift.BY, by = 5), cancellable = true)
	public void tick(CallbackInfo ci) {
		if(ModConfig.FIX_FURNACE_LAVA_BUCKET.get()) {
			if(this.contents[1] != null && this.contents[1].itemId == ItemType.bucketLava.id) {
				ci.cancel();
				this.contents[1] = new ItemInstance(ItemType.bucket);
			}			
		}
	}
	
	@Inject(method = "getFuelTime", at = @At("HEAD"), cancellable = true)
	private void getFuelTime(ItemInstance stack, CallbackInfoReturnable<Integer> cir) {
		if (stack == null) {
			cir.setReturnValue(0);
			return;
		}

		if (stack.getType().id == ModTile.COAL_BLOCK.id) {
			cir.setReturnValue(16000);
			return;
		}
	}

	/*
	 * Properly saving and loading the fuelTime as otherwise minecraft tries to
	 * get it from the fuel item slot, which can be empty...and so the fuel time
	 * defaults to 200. By saving it we make sure the fuel time is always
	 * correct regardless of the fuel slot contents.
	 */
	
	@Inject(method = "writeIdentifyingData", at = @At("TAIL"))
	public void writeIdentifyingData(CompoundTag tag, CallbackInfo ci) {
		tag.put("FuelTime", this.fuelTime);
	}

	@Inject(method = "readIdentifyingData", at = @At("TAIL"))
	public void readIdentifyingData(CompoundTag tag, CallbackInfo ci) {
		this.fuelTime = tag.getInt("FuelTime");
	}
}
